using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;
using BearWare;

namespace TeamTalk.NET
{
    class ChannelsView
    {
        TeamTalk4 ttclient;
        TreeView treeview;

        public enum ImageIndex
        {
            CHANNEL,
            USER
        }

        public ChannelsView(TeamTalk4 tt, TreeView tree)
        {
            ttclient = tt;
            treeview = tree;

            ttclient.OnCmdChannelNew += new TeamTalk4.ChannelUpdate(ttclient_OnCmdChannelNew);
            ttclient.OnCmdChannelUpdate += new TeamTalk4.ChannelUpdate(ttclient_OnCmdChannelUpdate);
            ttclient.OnCmdChannelRemove += new TeamTalk4.ChannelUpdate(ttclient_OnCmdChannelRemove);

            ttclient.OnCmdUserJoinedChannel += new TeamTalk4.UserUpdate(ttclient_OnCmdUserJoinedChannel);
            ttclient.OnCmdUserUpdate += new TeamTalk4.UserUpdate(ttclient_OnCmdUserUpdate);
            ttclient.OnCmdUserLeftChannel += new TeamTalk4.UserUpdate(ttclient_OnCmdUserLeftChannel);

            ttclient.OnUserTalking += new TeamTalk4.UserTalking(ttclient_OnUserTalking);
            ttclient.OnVoiceActivation += new TeamTalk4.VoiceActivation(ttclient_OnVoiceActivation);
        }

        public TreeNode GetTreeNode(int id, int img_type)
        {
            Queue<TreeNode> queue = new Queue<TreeNode>();
            foreach (TreeNode node in treeview.Nodes)
                queue.Enqueue(node);

            while (queue.Count > 0)
            {
                TreeNode node = queue.Dequeue();
                if (node.ImageIndex == (int)img_type)
                    if ((int)node.Tag == id)
                        return node;

                foreach (TreeNode n in node.Nodes)
                {
                    queue.Enqueue(n);
                }
            }
            return null;
        }

        public TreeNode GetChannel(int channelid)
        {
            return GetTreeNode(channelid, (int)ImageIndex.CHANNEL);
        }

        public TreeNode GetUser(int userid)
        {
            return GetTreeNode(userid, (int)ImageIndex.USER);
        }

        public int GetSelectedChannel()
        {
            if (treeview.SelectedNode != null &&
               treeview.SelectedNode.ImageIndex == (int)ImageIndex.CHANNEL)
                return (int)treeview.SelectedNode.Tag;
            return 0;
        }

        public int GetSelectedUser()
        {
            if (treeview.SelectedNode != null &&
               treeview.SelectedNode.ImageIndex == (int)ImageIndex.USER)
                return (int)treeview.SelectedNode.Tag;
            return 0;
        }

        void ttclient_OnCmdChannelNew(int nChannelID)
        {
            Channel chan;
            if (!ttclient.GetChannel(nChannelID, out chan))
                return;

            TreeNode parent = GetChannel(chan.nParentID);
            if (parent == null)
            {
                //it's the root channel

                //user servername as root channel name
                ServerProperties prop;
                ttclient.GetServerProperties(out prop);
                TreeNode newnode = new TreeNode(prop.szServerName, (int)ImageIndex.CHANNEL, (int)ImageIndex.CHANNEL);
                newnode.Tag = nChannelID;
                treeview.Nodes.Add(newnode);
            }
            else
            {
                TreeNode newnode = new TreeNode(chan.szName, (int)ImageIndex.CHANNEL, (int)ImageIndex.CHANNEL);
                newnode.Tag = nChannelID;
                parent.Nodes.Add(newnode);
            }
        }

        void ttclient_OnCmdChannelUpdate(int nChannelID)
        {
            Channel chan;
            if (!ttclient.GetChannel(nChannelID, out chan))
                return;

            if ((chan.uChannelType & ChannelType.CHANNEL_CLASSROOM) == ChannelType.CHANNEL_DEFAULT)
                return;

            TreeNode chanNode = GetChannel(nChannelID);
            Debug.Assert(chanNode != null);

            List<TreeNode> users = new List<TreeNode>();
            foreach (TreeNode userNode in chanNode.Nodes)
            {
                if (userNode.ImageIndex == (int)ImageIndex.USER)
                    users.Add(userNode);
            }
            foreach (TreeNode userNode in users)
            {
                User user;
                if (ttclient.GetUser((int)userNode.Tag, out user))
                {
                    bool voice = false, video = false;
                    foreach (int id in chan.voiceUsers)
                        voice |= (id == user.nUserID);
                    foreach (int id in chan.videoUsers)
                        video |= (id == user.nUserID);
                    userNode.Text = String.Format("{0} [Voice={1}, Video={2}]", user.szNickname, voice, video);
                }
            }
        }

        void ttclient_OnCmdChannelRemove(int nChannelID)
        {
            treeview.Nodes.Remove(GetChannel(nChannelID));
        }

        void ttclient_OnCmdUserJoinedChannel(int nUserID, int nChannelID)
        {
            TreeNode node = GetChannel(nChannelID);
            if (node == null)
                return;
            User user;
            if (!ttclient.GetUser(nUserID, out user))
                return;
            TreeNode newnode = new TreeNode(user.szNickname, (int)ImageIndex.USER, (int)ImageIndex.USER);
            newnode.Tag = nUserID;
            node.Nodes.Add(newnode);
        }

        void ttclient_OnCmdUserUpdate(int nUserID, int nChannelID)
        {
            TreeNode node = GetUser(nUserID);
            User user;
            if (node == null || !ttclient.GetUser(nUserID, out user))
                return;
            node.Text = user.szNickname;
        }

        void ttclient_OnCmdUserLeftChannel(int nUserID, int nChannelID)
        {
            treeview.Nodes.Remove(GetUser(nUserID));
        }

        void ttclient_OnUserTalking(int nUserID, bool bTalking)
        {
            TreeNode node = GetUser(nUserID);
            if(node != null)
                node.BackColor = bTalking ? Color.Green : Color.Transparent;
        }

        void ttclient_OnVoiceActivation(bool bVoiceActive)
        {
            TreeNode node = GetUser(ttclient.GetMyUserID());
            if (node != null)
                node.BackColor = bVoiceActive ? Color.Green : Color.Transparent;
        }
    }
}
